#!/usr/bin/env python
# _*_ coding: iso-8859-1 _*_



##import win32serviceutil as wsu
##import win32service as ws
##import win32api as wa
import logging
import socket
import time
import os

import reg
import option
import process
import lance_cmd
from pbgest import PBgest

input_template="""Windows Registry Editor Version 5.00

[HKEY_LOCAL_MACHINE\SOFTWARE\ORL\WinVNC3\Default]
"InputsEnabled"=dword:0000000%s

"""


class Vnc:
    """Gestion vnc distante. Actions possibles :
            - start  - restart  - start_listen  - connect (ip)
            - stop              - stop_listen   - disconnect
            - setinputs True/False (gestion clavier/souris)
    """
    def __init__(self, vncdir, tmpdir):
        self.vncdir = vncdir
        self.tmpdir = tmpdir
        self.vnc_conf = os.path.join(self.vncdir, 'eole_conf.vnc')
        self.sname = 'winvnc'
        self.winvnc_exe = os.path.join(self.vncdir, 'winvnc.exe')
        self.vncviewer = os.path.join(self.vncdir, 'vncviewer.exe')
        
    ###################
    ## Configuration ##
    ###################
    def setinputs(self, val=False, restart=True):
        """(ds)activer souris/clavier (activ par dfaut)
        """
        logging.debug("Application du type de controle 'setinputs' var=%s"%val)
        if val: t = '1'
        else: t = '0'
        reg_liste = [{ 'hkey' : 'HKEY_LOCAL_MACHINE', 'keypath' : r'SOFTWARE\ORL\WinVNC3\Default',
                       'keyname' : 'InputsEnabled', 'keytype' : 'REG_DWORD', 'value' : t}]
        reg.put_regs(reg_liste)
        if self.isrunning() and restart:
            self.restart()
        
    def set_type(self, type):
        """Renseigne le mode d'observation autorise dans l'EAD
        """
        reg_liste = [{ 'hkey' : 'HKEY_LOCAL_MACHINE', 'keypath' : 'Software\\Eole\\Scribe',
                       'keyname' : 'vnc_viewonly', 'keytype' : 'REG_SZ', 'value' : type}]
        return reg.put_regs(reg_liste)

    ################
    ## VNC Server ##
    ################
    def start(self):
        """Dmarre winvnc dans l'environnement de l'utilisateur (Vista oblige)
        """
        logging.debug('Tentative de demarrage de winvnc')
        if not self.isrunning():
            logging.debug('Demarrage de winvnc')
            lance_usercmd(self.winvnc_exe, nowait=True)
            # attendre 2 secondes que vnc dmarre
            for i in range(10):
                if self.isrunning():
                    return True
                time.sleep(0.2)
            return False

    def stop(self):
        """Mthode la plus sre (radicale) pour arrter "winvnc"
        """
        logging.debug("Tentative d'arret de winvnc")
        if self.isrunning():
            logging.debug('Arret de winvnc')
            process.killProcName('winvnc.exe')
    
    def restart(self):
        self.stop()
        self.start()
        
    def isrunning(self):
        """Y'a-t-il un procesus "winvnc.exe" ?
        """
        return process.isrunning('winvnc.exe')
    
    def connect(self, ip):
        """connecter une ip qui est en mode "listen"
        """
        self.start()
        lance_usercmd('%s -connect %s'%(self.winvnc_exe, ip), nowait=True)
    
    def disconnect(self):
        """redmarrer le service VNC dconnecte
        """
        self.restart()
    
    ################
    ## VNC Viewer ##
    ################
    def start_listen(self, args=None, conf=None):
        """se mettre en mode listen
        """
        if not args: args = ''
        if conf: file(self.vnc_conf, 'w').write(conf)
        if not self.is_listening():
            cmd = '%s /config %s /listen %s'%(self.vncviewer, self.vnc_conf, args)
            lance_cmd.lancecmd(cmd.strip(), nowait=True)

    def start_user_listen(self, args=None, conf=None):
        """se mettre en mode listen
        """
        if not args: args = ''
        if conf: file(self.vnc_conf, 'w').write(conf)
        if not self.is_listening():
            cmd = '%s /config %s /listen %s'%(self.vncviewer, self.vnc_conf, args)
            lance_usercmd(cmd.strip(), nowait=True)
        
    def stop_listen(self):
        """kill self.listen_pid"""
        process.killProcName('vncviewer.exe')
    
    def is_listening(self):
        """Test si le port 5400 ('vncviewer /listen') est ouvert
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(1)
            s.connect(('127.0.0.1', int(5400)))
            s.close()
            return True
        except:
            return False

    def view(self, ip, args=None, conf=None):
        """Mode  l'ancienne, utilise pour Win9x
        """
        if not args: args = ''
        if conf: file(self.vnc_conf, 'w').write(conf)
        cmd = '%s /config %s %s %s'%(self.vncviewer, self.vnc_conf, args, ip)
        lance_usercmd(cmd, nowait=True)

def lance_usercmd(cmd, hide=False, nowait=False, waitinput=False):
    pbgest = PBgest(ip='127.0.0.1', port=option.get_port_utilisateur())
    pbgest.d.addCallback(lambda object: object.callRemote('execute', cmd, hide, nowait, waitinput))
    return pbgest.d

